@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2018 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -13,6 +13,8 @@ from copy import deepcopy
 from string import Template
 
 from jsonschema import ValidationError, validate
+from reana_commons.config import KUBERNETES_MEMORY_FORMAT
+from reana_commons.utils import check_htcondor_max_runtime
 
 serial_workflow_schema = {
     "$schema": "http://json-schema.org/draft-06/schema#",
@@ -27,9 +29,7 @@ serial_workflow_schema = {
             "items": {
                 "$id": "#/properties/steps/items",
                 "type": "object",
-                "required": [
-                    "commands"
-                ],
+                "required": ["commands"],
                 "properties": {
                     "name": {
                         "$id": "#/properties/steps/properties/environment",
@@ -39,6 +39,11 @@ serial_workflow_schema = {
                         "$id": "#/properties/steps/properties/environment",
                         "type": "string",
                     },
+                    "unpacked_image": {
+                        "$id": "#/properties/steps/properties/unpacked_image",
+                        "type": "boolean",
+                        "default": "false",
+                    },
                     "compute_backend": {
                         "$id": "#/properties/steps/properties/compute_backend",
                         "type": "string",
@@ -49,24 +54,50 @@ serial_workflow_schema = {
                         "type": "boolean",
                         "default": "false",
                     },
+                    "kubernetes_uid": {
+                        "$id": "#/properties/steps/properties/kubernetes_uid",
+                        "type": "integer",
+                        "default": "None",
+                    },
+                    "kubernetes_memory_limit": {
+                        "$id": "#/properties/steps/properties/kubernetes_memory_limit",
+                        "type": "string",
+                        "default": "",
+                        "pattern": KUBERNETES_MEMORY_FORMAT,
+                    },
+                    "voms_proxy": {
+                        "$id": "#/properties/steps/properties/voms_proxy",
+                        "type": "boolean",
+                        "default": "false",
+                    },
+                    "htcondor_max_runtime": {
+                        "$id": "#/properties/steps/properties/htcondor_max_runtime",
+                        "type": "string",
+                        "default": "",
+                    },
+                    "htcondor_accounting_group": {
+                        "$id": "#/properties/steps/properties/htcondor_accounting_group",
+                        "type": "string",
+                        "default": "",
+                    },
                     "commands": {
                         "$id": "#/properties/steps/properties/commands",
                         "type": "array",
                         "minItems": 1,
                         "items": {
                             "$id": "#/properties/steps/items/properties/"
-                                "commands/items",
-                            "type": "string"
+                            "commands/items",
+                            "type": "string",
                         },
-                    }
-                }
-            }
+                    },
+                },
+            },
         }
-    }
+    },
 }
 
 
-def serial_load(workflow_file, specification, parameters=None, original=None):
+def serial_load(workflow_file, specification, parameters=None, original=None, **kwargs):
     """Validate and return a expanded REANA Serial workflow specification.
 
     :param workflow_file: A specification file compliant with
@@ -74,15 +105,16 @@ def serial_load(workflow_file, specification, parameters=None, original=None):
     :returns: A dictionary which represents the valid Serial workflow with all
         parameters expanded.
     """
+    if not check_htcondor_max_runtime(specification):
+        raise Exception("Invalid input in htcondor_max_runtime.")
+
     parameters = parameters or {}
 
     if not specification:
-        with open(workflow_file, 'r') as f:
+        with open(workflow_file, "r") as f:
             specification = json.loads(f.read())
 
-    expanded_specification = _expand_parameters(specification,
-                                                parameters,
-                                                original)
+    expanded_specification = _expand_parameters(specification, parameters, original)
 
     validate(specification, serial_workflow_schema)
 
@@ -90,11 +122,11 @@ def serial_load(workflow_file, specification, parameters=None, original=None):
 
 
 def _expand_parameters(specification, parameters, original=None):
-    """Expand parameters inside comands for Serial workflow specifications.
+    """Expand parameters inside commands for Serial workflow specifications.
 
     :param specification: Full valid Serial workflow specification.
     :param parameters: Parameters to be extended on a Serial specification.
-    :param original: Flag which, determins type of specifications to return.
+    :param original: Flag which, determines type of specifications to return.
     :returns: If 'original' parameter is set, a copy of the specification
         whithout expanded parametrers will be returned. If 'original' is not
         set, a copy of the specification with expanded parameters (all $varname
@@ -102,21 +134,23 @@ def _expand_parameters(specification, parameters, original=None):
         will be thrown if the parameters can not be expanded.
     :raises: jsonschema.ValidationError
     """
-    expanded_specification = deepcopy(specification)
-
-    try:
-        for step_num, step in enumerate(expanded_specification['steps']):
-            current_step = expanded_specification['steps'][step_num]
-            for command_num, command in enumerate(step['commands']):
-                current_step['commands'][command_num] = \
-                    Template(command).substitute(parameters)
-        # if call is done from client, original==True and original
-        # specifications withtout applied parameters are returned.
-        if original:
-            return specification
-        else:
+    # if call is done from client, original==True and original
+    # specifications withtout applied parameters are returned.
+    if original:
+        return specification
+    else:
+        try:
+            expanded_specification = deepcopy(specification)
+            for step_num, step in enumerate(expanded_specification["steps"]):
+                current_step = expanded_specification["steps"][step_num]
+                for command_num, command in enumerate(step["commands"]):
+                    current_step["commands"][command_num] = Template(
+                        command
+                    ).substitute(parameters)
             return expanded_specification
-    except KeyError as e:
-        raise ValidationError('Workflow parameter(s) could not '
-                              'be expanded. Please take a look '
-                              'to {params}'.format(params=str(e)))
+        except KeyError as e:
+            raise ValidationError(
+                "Workflow parameter(s) could not "
+                "be expanded. Please take a look "
+                "to {params}".format(params=str(e))
+            )
