@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of REANA.
-# Copyright (C) 2018 CERN.
+# Copyright (C) 2018, 2019, 2020, 2021 CERN.
 #
 # REANA is free software; you can redistribute it and/or modify it
 # under the terms of the MIT License; see LICENSE file for more details.
@@ -11,27 +11,31 @@
 import json
 import logging
 import os
+import platform
 import shutil
 import subprocess
 import time
 import uuid
 from hashlib import md5
+from pathlib import Path
 
 import click
 import requests
 
 from reana_commons.config import (
     CVMFS_REPOSITORIES,
+    HTCONDOR_JOB_FLAVOURS,
     REANA_COMPONENT_NAMING_SCHEME,
     REANA_COMPONENT_PREFIX,
     REANA_COMPONENT_TYPES,
     REANA_CVMFS_PVC_TEMPLATE,
     REANA_CVMFS_SC_TEMPLATE,
-    HTCONDOR_JOB_FLAVOURS,
+    REANA_JOB_CONTROLLER_CONNECTION_CHECK_SLEEP,
 )
+from reana_commons.errors import REANAMissingWorkspaceError
 
 
-def click_table_printer(headers, _filter, data):
+def click_table_printer(headers, _filter, data, colours=None):
     """Generate space separated output for click commands."""
     _filter = [h.lower() for h in _filter] + [h.upper() for h in _filter]
     header_indexes = [i for i, item in enumerate(headers)]
@@ -53,10 +57,13 @@ def click_table_printer(headers, _filter, data):
     formatted_output = "   ".join(formatted_output_parts)
     # Print the table with the headers capitalized
     click.echo(formatted_output.format(*[h.upper() for h in headers]))
-    for row in data:
+    colours = colours if len(colours or []) == len(data) else None
+    for i, row in enumerate(data):
         if header_indexes:
             row = [row[i] for i in header_indexes]
-        click.echo(formatted_output.format(*row))
+        click.secho(
+            formatted_output.format(*row), fg=colours[i] if colours else None,
+        )
 
 
 def calculate_hash_of_dir(directory, file_list=None):
@@ -109,6 +116,9 @@ def calculate_file_access_time(workflow_workspace):
     for subdir, dirs, files in os.walk(workflow_workspace):
         for file in files:
             file_path = os.path.join(subdir, file)
+            # skip symlinks
+            if os.path.islink(file_path):
+                continue
             access_times[file_path] = os.stat(file_path).st_atime
     return access_times
 
@@ -189,27 +199,130 @@ def build_caching_info_message(
     return caching_info_message
 
 
-def get_workspace_disk_usage(workspace, summarize=False, block_size=None):
-    """Retrieve disk usage information of a workspace."""
-    command = ["du"]
-    if block_size in ["b", "k"]:
-        command.append("-{}".format(block_size))
+def remove_upper_level_references(path):
+    """Remove upper than `./` references.
+
+    Collapse separators/up-level references avoiding references to paths
+    outside working directory.
+
+    :param path: User provided path to a file or directory.
+    :return: Returns the corresponding sanitized path.
+    """
+    return os.path.normpath("/" + path).lstrip("/")
+
+
+def is_directory(directory_path, path):
+    """Whether the given path matches a directory or not.
+
+    :param directory_path: Directory to check files from.
+    :param path: Optional wildcard pattern to use for the check.
+    :return: Full path if it is a directory, False if not.
+    """
+    secure_path = remove_upper_level_references(path)
+    full_path = Path(directory_path, secure_path)
+    if full_path.is_dir():
+        return full_path
+    return False
+
+
+def get_files_recursive_wildcard(directory_path, path):
+    """Get file(s) fitting the wildcard from the workspace.
+
+    :param directory_path: Directory to get files from.
+    :param path: Wildcard pattern to use for the extraction.
+    :return: list of paths sorted by length.
+    """
+    secure_path = remove_upper_level_references(path)
+    # if `secure_path` is a directory, append `/*` to get all the files inside
+    if is_directory(directory_path, secure_path):
+        _rstrip_path = secure_path.rstrip("/")
+        secure_path = "{}/*".format(_rstrip_path)
+    posix_dir_prefix = Path(directory_path)
+    paths = list(posix_dir_prefix.glob(secure_path))
+    # sort paths by length to start with the leaves of the directory tree
+    paths.sort(key=lambda path: len(str(path)), reverse=True)
+    return paths, posix_dir_prefix
+
+
+def get_disk_usage_info_paths(absolute_path, command, name_filter):
+    """Retrieve the path for disk usage information.
+
+    :param absolute_path: System path to reana filesystem.
+    :param command: Command to get the disk usage from reana filesystem.
+    :param name_filter: Name filter parameters if any.
+
+    :return: List of disk usage info containing the file path and size.
+    """
+    if name_filter:
+        path_list = []
+        for _path in name_filter:
+            paths, _ = get_files_recursive_wildcard(absolute_path, _path)
+            if paths:
+                path_list += paths
+        if path_list:
+            for path in path_list:
+                command.append(path)
+            disk_usage_info = subprocess.check_output(command).decode().split()
+        else:
+            disk_usage_info = []
     else:
-        command.append("-h")
+        command.append(absolute_path)
+        disk_usage_info = subprocess.check_output(command).decode().split()
+    return disk_usage_info
+
+
+def get_disk_usage(
+    directory, summarize=False, search=None, to_human_readable_units=None
+):
+    """Retrieve directory disk usage information.
+
+    :param directory: Disk usage directory.
+    :param summarize: Displays a total size of a directory.
+    :param search: Filter parameters to show only files that match certain filtering.
+    :param to_human_readable_units: Callback to transform bytes to human
+        readable units.
+
+    :return: List of dicts with file name and size.
+    """
+    if not os.path.exists(directory):
+        raise REANAMissingWorkspaceError("Directory does not exist.")
+    command = ["du"]
     if summarize:
         command.append("-s")
     else:
         command.append("-a")
-    command.append(workspace)
-    disk_usage_info = subprocess.check_output(command).decode().split()
-    # create pairs of (size, filename)
-    filesize_pairs = list(zip(disk_usage_info[::2], disk_usage_info[1::2]))
-    filesizes = []
-    for filesize_pair in filesize_pairs:
-        size, name = filesize_pair
-        # trim workspace path in every file name
-        filesizes.append({"name": name[len(workspace) :], "size": size})
-    return filesizes
+    if "Darwin" not in platform.system():
+        # Default block size in GNU is KB
+        command.append("-b")
+
+    name_filter = None
+    size_filter = None
+    if search:
+        search = json.loads(search)
+        name_filter = search.get("name")
+        size_filter = search.get("size")
+
+    disk_usage_info = get_disk_usage_info_paths(directory, command, name_filter)
+    if disk_usage_info:
+        filesize_pairs = list(zip(disk_usage_info[::2], disk_usage_info[1::2]))
+        filesizes = []
+        for filesize_pair in filesize_pairs:
+            size, name = filesize_pair
+            size = int(size)
+            # trim workspace path in every file name, and transform bytes if necessary
+            file_data = {
+                "name": name[len(directory) :],
+                "size": {"raw": size},
+            }
+            if to_human_readable_units:
+                file_data["size"]["human_readable"] = to_human_readable_units(size)
+            if size_filter:
+                if str(size) in size_filter:
+                    filesizes.append(file_data)
+            else:
+                filesizes.append(file_data)
+        return filesizes
+    return disk_usage_info
 
 
 def render_cvmfs_pvc(cvmfs_volume):
@@ -280,7 +393,7 @@ def check_connection_to_job_controller(port=5000):
                 break
         except Exception:
             pass
-        time.sleep(10)
+        time.sleep(REANA_JOB_CONTROLLER_CONNECTION_CHECK_SLEEP)
         retry_counter += 1
     else:
         logging.error("Job controller is not reachable.", exc_info=True)
